@@ -110,83 +110,119 @@ class ConfigurationOption(optparse.Option):
     """option that keeps track if it is seen"""
     # TODO: this should be configurable or something
     def take_action(self, action, dest, opt, value, values, parser):
-        """add the parsed option to the set of things parsed"""
         optparse.Option.take_action(self, action, dest, opt, value, values, parser)
+
+        # add the parsed option to the set of things parsed
         if not hasattr(parser, 'parsed'):
             parser.parsed = set()
         parser.parsed.add(dest)
 
+        # switch on types
+        formatter = getattr(parser, 'cli_formatter')
+        if formatter:
+            formatter = formatter(dest)
+            setattr(values, dest, formatter(getattr(values, dest)))
+
 ### plugins for option types
 ### TODO: this could use a bit of thought
 ### They should probably be classes
-def base_cli(name, value):
-    # CLI arguments
-    args = value.get('flags', ['--%s' % name])
-    if not args:
-        # No CLI interface
-        return (), {}
+# class Option(object):
+#     def arguments(self, name, value):
+#         """return arguments appropriate for construction of an optparse.Option
+#     def take_action(self, ...):
+#         """do something appropriate based on type"""
 
-    kw = {'dest': name}
-    help = value.get('help', name)
-    if 'default' in value:
-        kw['default'] = value['default']
-        # TODO: use default pattern a la
-        # - http://hg.mozilla.org/build/talos/file/c6013a2f09ce/talos/PerfConfigurator.py#l358
-        # - http://k0s.org/mozilla/hg/bzconsole/file/d5e88dadde69/bzconsole/command.py#l12
+class BaseCLI(object):
+    """base_cli for all option types"""
 
-        help += ' [DEFAULT: %s]' % value['default']
-    kw['help'] = help
-    kw['action'] = 'store'
-    return args, kw
+    def __call__(self, name, value):
+        """return args, kwargs needed to instantiate an optparse option"""
 
-def bool_cli(name, value):
+        args = value.get('flags', ['--%s' % name])
+        if not args:
+            # No CLI interface
+            return (), {}
 
-    # preserve the default values
-    help = value.get('help')
-    flags = value.get('flags')
+        kw = {'dest': name}
+        help = value.get('help', name)
+        if 'default' in value:
+            kw['default'] = value['default']
+            help += ' [DEFAULT: %s]' % value['default']
+        kw['help'] = help
+        kw['action'] = 'store'
+        return args, kw
 
-    args, kw = base_cli(name, value)
-    kw['help'] = help # reset
-    if value.get('default'):
-        kw['action'] = 'store_false'
-        if not flags:
-            args = ['--no-%s' % name]
-        if not help:
-            kw['help'] = 'disable %s' % name
-    else:
-        kw['action'] = 'store_true'
-        if not help:
-            kw['help'] = 'enable %s' % name
-    return args, kw
+    def take_action(self, value):
+        return value
 
-def list_cli(name, value):
-    args, kw = base_cli(name, value)
+class BoolCLI(BaseCLI):
 
-    # TODO: could use 'extend'
-    # - http://hg.mozilla.org/build/mozharness/file/5f44ba08f4be/mozharness/base/config.py#l41
+    def __call__(self, name, value):
 
-    # TODO: what about nested types?
-    kw['action'] = 'append'
-    return args, kw
+        # preserve the default values
+        help = value.get('help')
+        flags = value.get('flags')
 
-def int_cli(name, value):
-    args, kw = base_cli(name, value)
-    kw['type'] = 'int'
-    return args, kw
+        args, kw = BaseCLI.__call__(self, name, value)
+        kw['help'] = help # reset
+        if value.get('default'):
+            kw['action'] = 'store_false'
+            if not flags:
+                args = ['--no-%s' % name]
+            if not help:
+                kw['help'] = 'disable %s' % name
+        else:
+            kw['action'] = 'store_true'
+            if not help:
+                kw['help'] = 'enable %s' % name
+        return args, kw
 
-def float_cli(name, value):
-    args, kw = base_cli(name, value)
-    kw['type'] = 'float'
-    return args, kw
+class ListCLI(BaseCLI):
+
+    def __call__(self, name, value):
+        args, kw = BaseCLI.__call__(self, name, value)
+
+        # TODO: could use 'extend'
+        # - http://hg.mozilla.org/build/mozharness/file/5f44ba08f4be/mozharness/base/config.py#l41
+
+        # TODO: what about nested types?
+        kw['action'] = 'append'
+        return args, kw
+
+class IntCLI(BaseCLI):
+
+    def __call__(self, name, value):
+        args, kw = BaseCLI.__call__(self, name, value)
+        kw['type'] = 'int'
+        return args, kw
+
+class FloatCLI(BaseCLI):
+
+    def __call__(self, name, value):
+        args, kw = BaseCLI.__call__(self, name, value)
+        kw['type'] = 'float'
+        return args, kw
+
+class DictCLI(ListCLI):
+
+    delimeter = '='
+
+    def take_action(self, value):
+        bad = [i for i in value if self.delimeter not in i]
+        if bad:
+            raise AssertionError("Each value must be delimited by '%s': %s" % (self.delimeter, bad))
+        return dict([i.split(self.delimeter, 1) for i in value])
 
 # TODO: 'dict'-type cli interface
 
-types = {bool:  bool_cli,
-         int:   int_cli,
-         float: float_cli,
-         list:  list_cli,
-         None:  base_cli} # default
-__all__ += [i.__name__ for i in types.values()]
+types = {bool:  BoolCLI(),
+         int:   IntCLI(),
+         float: FloatCLI(),
+         list:  ListCLI(),
+         dict:  DictCLI(),
+         None:  BaseCLI()} # default
+
+__all__ += [i.__class__.__name__ for i in types.values()]
 
 class Configuration(optparse.OptionParser):
     """declarative configuration object"""
@@ -264,7 +300,7 @@ class Configuration(optparse.OptionParser):
     def validate(self):
         """validate resultant configuration"""
 
-        for key, value in self.options.items():
+        for key, value in self.items():
             if key not in self.config:
                 required = value.get('required')
                 if required:
@@ -286,7 +322,7 @@ class Configuration(optparse.OptionParser):
             self.add(config)
 
         # add defaults if not present
-        for key, value in self.options.items():
+        for key, value in self.items():
             if 'default' in value and key not in self.config:
                 self.config[key] = value['default']
 
@@ -304,6 +340,10 @@ class Configuration(optparse.OptionParser):
     ### methods for optparse
     ### XXX could go in a subclass
 
+    def cli_formatter(self, option):
+        handler = self.types[self.options[option].get('type')]
+        return getattr(handler, 'take_action', lambda x: 1)
+
     def optparse_options(self, parser):
         """add optparse options to a OptionParser instance"""
         for key, value in self.items():
@@ -315,6 +355,8 @@ class Configuration(optparse.OptionParser):
             parser.add_option(*args, **kw)
 
     def parse_args(self, *args, **kw):
+
+        self.parsed = set()
         options, args = optparse.OptionParser.parse_args(self, *args, **kw)
 
         # get CLI configuration options
